"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../utils/constants");
require("isomorphic-fetch");
const debug = require('debug')('graphcool');
function sendGraphQLRequest(queryString, resolver, variables) {
    return __awaiter(this, void 0, void 0, function* () {
        const configContents = resolver.read(constants_1.graphcoolConfigFilePath);
        const { token } = JSON.parse(configContents);
        const queryVariables = variables || {};
        const payload = {
            query: queryString,
            variables: queryVariables
        };
        debug(`Send request to ${constants_1.systemAPIEndpoint}\nPayload: \n${JSON.stringify(payload)}`);
        const result = yield fetch(constants_1.systemAPIEndpoint, {
            method: 'POST',
            headers: {
                'Authorization': `Bearer ${token}`,
                'Content-Type': 'application/json'
            },
            body: JSON.stringify(payload),
        });
        const json = yield result.json();
        if (process.env.DEBUG === 'graphcool') {
            debug(`Received JSON response: \n${JSON.stringify(json)}`);
        }
        return json;
    });
}
function createProject(name, schema, resolver, alias, region) {
    return __awaiter(this, void 0, void 0, function* () {
        const mutation = `\
mutation addProject($schema: String!, $name: String!, $alias: String, $region: Region) {
  addProject(input: {
    name: $name,
    schema: $schema,
    alias: $alias,
    region: $region,
    clientMutationId: "static"
  }) {
    project {
      id
      name
      schema
      alias
      version
    }
  }
}
`;
        let variables = { name, schema };
        if (alias) {
            variables = Object.assign({}, variables, { alias });
        }
        if (region) {
            variables = Object.assign({}, variables, { region });
        }
        const json = yield sendGraphQLRequest(mutation, resolver, variables);
        if (!json.data.addProject) {
            throw json;
        }
        const projectId = json.data.addProject.project.id;
        const version = json.data.addProject.project.version;
        schema = json.data.addProject.project.schema;
        const returnedAlias = json.data.addProject.project.alias;
        name = json.data.addProject.project.name;
        const projectInfo = { projectId, version, alias: returnedAlias, schema, name };
        return projectInfo;
    });
}
exports.createProject = createProject;
function pushNewSchema(newSchema, force, resolver) {
    return __awaiter(this, void 0, void 0, function* () {
        const mutation = `\
 mutation($newSchema: String!, $force: Boolean!) {
  migrateProject(input: {
    newSchema: $newSchema,
    force: $force
  }) {
    migrationMessages {
      type
      action
      name
      description
      subDescriptions {
        type
        action
        name
        description
      }
    }
    errors {
      description
      type
      field
    }
    project {
      id
      name
      schema
      alias
      version
    }
  }
}
`;
        const variables = {
            newSchema,
            force
        };
        const json = yield sendGraphQLRequest(mutation, resolver, variables);
        if (!json.data.migrateProject) {
            throw json;
        }
        const messages = json.data.migrateProject.migrationMessages;
        const errors = json.data.migrateProject.errors;
        const newVersion = json.data.migrateProject.project.version;
        newSchema = json.data.migrateProject.project.schema;
        const migrationResult = { messages, errors, newVersion, newSchema };
        return migrationResult;
    });
}
exports.pushNewSchema = pushNewSchema;
function statusMessage(newSchema, resolver) {
    return __awaiter(this, void 0, void 0, function* () {
        const mutation = `\
mutation($newSchema: String!) {
  migrateProject(input: {
    newSchema: $newSchema,
    isDryRun: true
  }) {
    migrationMessages {
      type
      action
      name
      description
      subDescriptions {
        type
        action
        name
        description
      }
    }
    errors {
      description
      type
      field
    }
    project {
        id
        name
        schema
        alias
        version
    }
  }
}`;
        const variables = {
            newSchema,
        };
        const json = yield sendGraphQLRequest(mutation, resolver, variables);
        if (!json.data.migrateProject) {
            throw json;
        }
        const messages = json.data.migrateProject.migrationMessages;
        const errors = json.data.migrateProject.errors;
        const newVersion = json.data.migrateProject.project.version;
        newSchema = json.data.migrateProject.project.schema;
        const migrationResult = { messages, errors, newVersion, newSchema };
        return migrationResult;
    });
}
exports.statusMessage = statusMessage;
function fetchProjects(resolver) {
    return __awaiter(this, void 0, void 0, function* () {
        const query = `\
{
  viewer {
	  user {
      projects {
        edges {
          node {
            id
            name
            schema
            alias
            version
          }
        }
      }
    }
  }
}`;
        const json = yield sendGraphQLRequest(query, resolver); // await fetch(systemAPIEndpoint, {
        const projects = json.data.viewer.user.projects.edges.map(edge => edge.node);
        const projectInfos = projects.map(p => ({
            projectId: p.id,
            name: p.name,
            alias: p.alias,
            schema: p.schema,
            version: p.version,
        }));
        return projectInfos;
    });
}
exports.fetchProjects = fetchProjects;
function pullProjectInfo(projectId, resolver) {
    return __awaiter(this, void 0, void 0, function* () {
        const query = `\
query ($projectId: ID!){
  viewer {
    project(id: $projectId) {
      id
      name
      alias
      schema
      version
    }
  }
}`;
        const variables = { projectId };
        const json = yield sendGraphQLRequest(query, resolver, variables);
        if (!json.data.viewer.project) {
            throw json;
        }
        const projectInfo = {
            projectId: json.data.viewer.project.id,
            name: json.data.viewer.project.name,
            schema: json.data.viewer.project.schema,
            alias: json.data.viewer.project.alias,
            version: String(json.data.viewer.project.version),
        };
        return projectInfo;
    });
}
exports.pullProjectInfo = pullProjectInfo;
function exportProjectData(projectId, resolver) {
    return __awaiter(this, void 0, void 0, function* () {
        const mutation = `\
mutation ($projectId: String!){
  exportData(input:{
    projectId: $projectId,
    clientMutationId: "asd"
  }) {
    url
  }
}`;
        const variables = { projectId };
        const json = yield sendGraphQLRequest(mutation, resolver, variables);
        if (!json.data.exportData) {
            throw json;
        }
        return json.data.exportData.url;
    });
}
exports.exportProjectData = exportProjectData;
function cloneProject(projectId, clonedProjectName, includeMutationCallbacks, includeData, resolver) {
    return __awaiter(this, void 0, void 0, function* () {
        const mutation = `\
mutation ($projectId: String!, $name: String!, $includeMutationCallbacks: Boolean!, $includeData: Boolean!){
  cloneProject(input:{
    name: $name,
    projectId: $projectId,
    includeData: $includeData,
    includeMutationCallbacks: $includeMutationCallbacks,
    clientMutationId: "asd"
  }) {
    project {
      id
      name
      alias
      schema
      version
    }
  }
}`;
        const variables = { projectId, name: clonedProjectName, includeMutationCallbacks, includeData };
        const json = yield sendGraphQLRequest(mutation, resolver, variables);
        if (!json.data.cloneProject) {
            throw json;
        }
        const projectInfo = {
            projectId: json.data.cloneProject.project.id,
            name: json.data.cloneProject.project.name,
            schema: json.data.cloneProject.project.schema,
            alias: json.data.cloneProject.project.alias,
            version: String(json.data.cloneProject.project.version),
        };
        return projectInfo;
    });
}
exports.cloneProject = cloneProject;
function parseErrors(response) {
    const errors = response.errors.map(error => ({
        message: error.message,
        requestId: error.requestId,
        code: String(error.code)
    }));
    return errors;
}
exports.parseErrors = parseErrors;
function generateErrorOutput(apiErrors) {
    const lines = apiErrors.map(error => `${error.message} (Request ID: ${error.requestId})`);
    const output = `\n${lines.join('\n')}\n\n${constants_1.contactUsInSlackMessage}`;
    return output;
}
exports.generateErrorOutput = generateErrorOutput;
//# sourceMappingURL=api.js.map